/*
 * $Id: kom-types.h,v 0.5 1992/01/31 18:22:26 ceder Exp $
 * Copyright (C) 1991  Lysator Academic Computer Association.
 *
 * This file is part of the LysKOM server.
 * 
 * LysKOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; either version 1, or (at your option) 
 * any later version.
 * 
 * LysKOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with LysKOM; see the file COPYING.  If not, write to
 * Lysator, c/o ISY, Linkoping University, S-581 83 Linkoping, SWEDEN,
 * or the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, 
 * MA 02139, USA.
 *
 * Please mail bug reports to bug-lyskom@lysator.liu.se. 
 */
/*
 *  KOM-types.h  --  Types used by both server and client of LysKOM
 *
 *
 *  Copyright (C) 1990	Lysator Computer Club,
 *			Linkoping University,  Sweden
 *
 *  Everyone is granted permission to copy, modify and redistribute
 *  this code, provided the people they give it to can.
 *
 *
 *  Filecreator: Thomas Bellman
 *		  Lysator Computer Club
 *		  Linkoping University
 *		  Sweden
 *
 *  email:	  Bellman@Lysator.LiU.SE
 */



#ifndef  KOM_TYPES_H_ALREADY_INCLUDED__

#define  KOM_TYPES_H_ALREADY_INCLUDED__

#ifndef SERVER
# ifndef CLIENT
#  error must define SERVER or CLIENT
# endif
#else
# ifdef CLIENT
#  error must not define both SERVER and CLIENT
# endif
#endif /* !SERVER */

#include <sys/types.h>
#include <limits.h>
#include <time.h>

#include <s-string.h>
#include "misc-types.h"


typedef		u_short 	Pers_no;
typedef		u_short		Conf_no;
typedef 	u_long		Text_no;
typedef		u_long		Local_text_no;
typedef		u_long		Session_no;

#ifdef PROT_a

/*
 * To uniquely identify a person, conference or text, three numbers
 * are used: Universe, Site and X_{recpt,person,text}. The universe is
 * omitted when it is LysKOM, or is implied by a echo conference.
 */
/*
 * LysKOM, RFC-822, FidoNet, News are examples of universes.
 */
/*typedef		u_long		Universe;*/

/*
 * JAKOM, lysator.liu.se, 2:504/113, foo.bar.com are examples of sites
 * in the above universes. (Note that comp.lang.c is _not_ a site in
 * the News universe).
 */
/*typedef		u_long		Site;*/

/*
 * Fritt Forum, ceder, Per Cederqvist, comp.lang.c are examples of
 * x_recpt in the above universes.
 */

/*
 * But for now, we have a System_id (sum of Universe and Site) and a
 * Global_x.
 */

typedef		u_long		System_id;
typedef		u_long		Global_recpt;
typedef		u_long		Global_person;
typedef		u_long		Global_text;

#endif


#define	MAX_PERS_NO		((Pers_no) USHRT_MAX)
#define MAX_CONF_NO		((Conf_no) USHRT_MAX)
#define MAX_TEXT_NO		((Text_no) ULONG_MAX)
#define	MAX_LOCAL_TEXT_NO	((Local_text_no) ULONG_MAX)



#ifdef  SERVER
	#define		PASSWD_LEN	64
	typedef 	char		Password[PASSWD_LEN];
	typedef		off_t		Text_index;
	typedef		time_t		Time;
#  define NULL_TIME_i	((Time) 0)

#else	/* CLIENT */
	typedef		struct tm	Time;
#  define NULL_TIME_i	((Time) { 0 })

#endif



/*
 * The privilige bits:  says what priviliges a person has.  E g
 *  if he is allowed to read texts he normally shouldn't be allowed
 *  to read. See file doc/security-levels.txt
 */
typedef	struct {
	u_int	wheel	: 1;
	u_int	admin	: 1;
	u_int	statistic : 1;
	u_int	create_pers: 1;
	u_int	create_conf: 1;
	u_int	change_name: 1;
	u_int	extern_gw : 1;
	u_int	flg8	: 1;	/* For future use. */
	u_int	flg9    : 1;
	u_int	flg10   : 1;
	u_int	flg11   : 1;
	u_int	flg12	: 1;
	u_int	flg13	: 1;
	u_int	flg14	: 1;
	u_int	flg15	: 1;
	u_int	flg16	: 1;
} Priv_bits;

#ifdef SERVER
#	define DEFAULT_PRIV_BITS_i	((Priv_bits)		\
	{	0, 0, 0, ANYONE_CAN_CREATE_NEW_PERSONS,		\
		ANYONE_CAN_CREATE_NEW_CONFS,			\
		DEFAULT_CHANGE_NAME, 0, 0, 0, 0, 0,		\
		0, 0, 0, 0, 0					\
	})
#else
#	define DEFAULT_PRIV_BITS_i	((Priv_bits)		\
	{	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,		\
		0, 0, 0, 0, 0					\
	})
#endif
extern  const Priv_bits		DEFAULT_PRIV_BITS;



/*
 * Flags in the Person struct.
 */
typedef struct {
    u_int  unread_is_secret : 1;/* FALSE if everyone is allowed to ask how */
    				/* many unread texts you have.	*/
    u_int  flg2	: 1;
    u_int  flg3	: 1;
    u_int  flg4	: 1;
    u_int  flg5	: 1;
    u_int  flg6	: 1;
    u_int  flg7	: 1;
    u_int  flg8	: 1;
} Personal_flags;

#define DEFAULT_PERSONAL_FLAGS_i	((Personal_flags)		\
					 { 0, 0, 0, 0, 0, 0, 0, 0 })
extern const Personal_flags DEFAULT_PERSONAL_FLAGS;



/* See file doc/misc_items */
typedef enum {
    recpt     = 0,	/* 0 Recipient (conference) */
    cc_recpt  = 1,	/* 1 Carbon Copy recipient (extra kopia) */
    comm_to   = 2,	/* 2 Comment to (text) */
    comm_in   = 3,	/* 3 Commented in (text) */
    footn_to  = 4,	/* 4 This is a footnote to (text) */
    footn_in  = 5,	/* 5 Footnote to this text in (text) */
#ifdef PROT_a
    x_author  = 10,	/* 10 External author (Global_name). */
#endif
    
    loc_no    = 6,	/* 6 Sequence number within conference */
    rec_time  = 7,	/* 7 Received at (time) */
    sent_by   = 8,	/* 8 Sent by (person) */
    sent_at   = 9	/* 9 Sent at (time) */
#ifdef PROT_a
	,
    x_person  = 11,	/* 11 External person id. (Global_name). */
    x_recpt   = 12,	/* 12 External recipient. (Global_name). */
    x_text    = 13,	/* 13 External text. (Global_text). */
    x_system  = 14	/* 14 External system identification (System_id). */
#endif
} Info_type;

#ifdef PROT_a
typedef String Global_name;
#endif

typedef	union {
		Conf_no		recipient;
		Conf_no		cc_recipient;
		Text_no		comment_to;
		Text_no		commented_in;
		Text_no		footnote_to;
		Text_no		footnoted_in;

#ifdef PROT_a
		Global_name	x_author; /* Protocol a. */
#endif

		Local_text_no	local_no;
		Time		received_at;
		Pers_no		sender;
		Time		sent_at;

#ifdef PROT_a
		Global_name	global_person;
		Global_name	global_recpt;
		Global_text	global_text;
		System_id	system_id;
#endif
} Info_datum;


/*  This struct contains miscellaneous information about a text.  */
typedef	struct {
	Info_type	type;
	Info_datum	datum;
} Misc_info;


/*  Fields of this type is supposed to tell the garbage collector
 *  which texts it should remove first.
 */
typedef u_long	Garb_nice;


/*  Struct for text status  */
typedef struct {
	Time		  creation_time;
#ifdef SERVER
        long              file_pos; /* Start of the text in the text file. */
#endif
	Pers_no		  author;
	u_short		  no_of_lines;
	String_size	  no_of_chars;
	u_short		  no_of_marks;	/* Antal markeringar */
	u_short		  no_of_misc;	/* Recipients, times, comments, ... */
	Misc_info	* misc_items;	/* List of miscellaneous info. */
} Text_stat;

#ifdef SERVER
#define	EMPTY_TEXT_STAT_i	((Text_stat)				\
				 {  NULL_TIME_i, 0, 0, 0, 0, 0, 0, NULL  })
#else
#define	EMPTY_TEXT_STAT_i	((Text_stat)				\
				 {  NULL_TIME_i, 0, 0, 0, 0, 0, NULL  })
#endif

extern const Text_stat EMPTY_TEXT_STAT;



typedef	struct {
	u_int		rd_prot	: 1;	/* !(Can anyone become a member?) */
	u_int		original : 1;	/* Comments forbidden? */
	u_int		secret : 1;	/* Secret conference? */
					/* Note: if a letterbox is secret it
					 * will be very hard for that person
					 * to log in, since he can't map his
					 * name to a pers_no. He must either
					 * know his pers_no, or have another
					 * identity which is his supervisor.
					 */
	u_int		letter_box : 1;	/* Is this a letter box? */
} Conf_type;

#define	NULL_CONF_TYPE_i	((Conf_type) {  0, 0, 0, 0  })
extern  const Conf_type	NULL_CONF_TYPE;



typedef struct {
	Pers_no		member;
} Member;



/* Struct for marks */
typedef struct {
	Text_no		text_no;
	u_char		mark_type;	/* It's up to the clients to decide the
					   meaning of this field. */
} Mark;



/*  Information about a person's membership in a conference  */
typedef struct {
	Conf_no		conf_no;
	u_char		priority;	/* Interrupt priority */
	Time		last_time_read;	/* Updated every time a text in this
					   conf. is marked as read.	*/
	Local_text_no	last_text_read;	/* All texts before and inclusive this
					   are read	*/
	u_short		no_of_read;
	Local_text_no * read_texts;	/* Texts after last_text_read. Sorted
					   in ascending order */
} Membership;

#define	EMPTY_MEMBERSHIP_i	((Membership)		\
			{  0, 0, NULL_TIME_i, 0, 0, NULL  })
extern  const Membership EMPTY_MEMBERSHIP;



/* Some structs to handle variable-sized arrays. */

typedef struct {
	u_short	  no_of_marks;
	Mark	* marks;		/* Pointer to an array of marks. */
} Mark_list;

#define	EMPTY_MARK_LIST_i	((Mark_list) {  0, NULL  })
extern const Mark_list EMPTY_MARK_LIST;


typedef struct {
    u_short no_of_confs;
    Conf_no  *conf_nos;
} Conf_no_list;

#define EMPTY_CONF_NO_LIST_i	((Conf_no_list) { 0, NULL  })
extern const Conf_no_list EMPTY_CONF_NO_LIST;
   

typedef struct {
	u_short	  no_of_members;
	Member	* members;
} Member_list;

#define EMPTY_MEMBER_LIST_i	((Member_list) {  0, NULL  })
extern const Member_list EMPTY_MEMBER_LIST;



typedef  struct {
	Local_text_no	  first_local_no;
    	u_long		  no_of_texts;
	Text_no		* texts;
}  Text_list;

#define EMPTY_TEXT_LIST_i	((Text_list) {  1, 0, NULL  })
extern const Text_list	EMPTY_TEXT_LIST;



/*  A list of conference numbers, also telling if it is a
 *  mailbox (i e a person) or an ordinary conference.	*/
/* Delete this ugly variant as soon as possible! */
typedef struct {
	u_long		  no_of_conf_nos;
	Conf_no		* conf_nos;
	Conf_type	* type_of_conf;
} Conf_list_old;
/* This is the way it should look! */
typedef struct {
	Conf_no   conf_no;
	Conf_type type;
} Micro_conf;

typedef struct {
	u_long       no_of_confs;
	Micro_conf * confs;
} Conf_list;


#define EMPTY_CONF_LIST_OLD_i	((Conf_list_old) {  0, NULL, NULL  })
extern const Conf_list_old	EMPTY_CONF_LIST_OLD;

#define EMPTY_CONF_LIST_i	((Conf_list) {  0, NULL  })
extern const Conf_list		EMPTY_CONF_LIST;


/*  A list of person numbers  */
typedef struct {
	u_long		  no_of_persons;
	Pers_no		* persons;
} Pers_list;

#define EMPTY_PERS_LIST_i	((Pers_list) {  0, NULL  })
extern  const Pers_list		EMPTY_PERS_LIST;



typedef struct {
	u_short	      no_of_confs;
	Membership  * confs;
} Membership_list;

#define EMPTY_MEMBERSHIP_LIST_i		((Membership_list) {  0, NULL  })
extern  const Membership_list		EMPTY_MEMBERSHIP_LIST;



/* The Info struct */
typedef struct {
	long		version;
	Conf_no		conf_pres_conf; /* Presentation of new confs */
	Conf_no		pers_pres_conf;	/* Presentation of new persons */
	Conf_no		motd_conf; 	/* Conf that receive motds */
	Conf_no		kom_news_conf;  /* News about kom */
	Text_no		motd_of_lyskom; /* To be displayed after login */
	/* and maybe more... */
} Info;



typedef	struct {
	Pers_no		creator;
	Time		creation_time;
	Text_no		presentation;
	Conf_no		supervisor;	/* Organisat|r f|r m|tet */
	Conf_no		permitted_submitters;	/* People who are allowed
					   to submit texts to this conf. 
					   0 -> anyone may submit. */
	Conf_no		super_conf;	/* Send unallowed submissions to
					   the super_conf. 
					   0 -> unallowed submissions bounce */
	Conf_type	type;		/* secret, rd_prot etc */
	Time		last_written;	/* Time of latest text in this conf. */
	Text_no		msg_of_day;	/* Message to be displayed when this
					   conf is referenced by the user. */
	Garb_nice	nice;		/* How long do texts in this
					   conf live? */
	String		name;		/* Name of conference */
#ifdef CLIENT
	u_short		no_of_members;
	Local_text_no	first_local_no;
	u_long		no_of_texts;
#else
	Member_list	members;	/* List of members in conf */
	Text_list	texts;		/* List of texts */
#endif
} Conference;


#ifdef CLIENT
#	define	EMPTY_CONFERENCE_i	((Conference)		\
	{	0, NULL_TIME_i, 0, 0, 0, 0, NULL_CONF_TYPE_i,	\
		NULL_TIME_i, 0, 0, EMPTY_STRING_i, 0, 0, 0	\
	})
#else	/* SERVER */
#	define	EMPTY_CONFERENCE_i	((Conference)		\
	{	0, NULL_TIME_i, 0, 0, 0, 0, NULL_CONF_TYPE_i,	\
		NULL_TIME_i, 0, 0, EMPTY_STRING_i,		\
		EMPTY_MEMBER_LIST_i, EMPTY_TEXT_LIST_i		\
	})
#endif	/* SERVER */
extern const Conference	EMPTY_CONFERENCE;


#ifdef SERVER
typedef struct {
	String		name;		/* Name of conference */
	Conf_type	type;		/* secret, rd_prot etc */
	Local_text_no	highest_local_no; /* highest local text no */
	Garb_nice	nice;	        /* Number of days to live */
} Small_conf;

#define EMPTY_SMALL_CONF_i ((Small_conf){EMPTY_STRING_i, NULL_CONF_TYPE_i, \
					     0, 0});

extern const Small_conf EMPTY_SMALL_CONF;
#endif	
	

/*  Struct for persons  */
typedef struct {
	Text_no		user_area;	/* Misc info the clients might want to
					   store. 0 = not used. */
	Priv_bits	privileges;
	Personal_flags	flags;
	Time		last_login; 	/* Or logout */
	u_long		total_time_present; /* Number of seconds. */
	u_long		sessions;	/* Number of sessions */
	u_long		created_lines;	/* No. of created lines (statistics) */
	u_long		created_bytes;	/* No. of created bytes (statistics) */
	u_long		read_texts;	/* No. of read texts (statistics) */
	u_long		no_of_text_fetches; /* (statistics) */
	u_short		created_persons; /* (statistics) */
	u_short		created_confs;	/* (statistics) */
	String		username;	/* User-name & hostname */
#ifdef CLIENT
	Local_text_no	first_created_text; /* The first text that still
					     * exists. */
	u_long		no_of_created_texts;
	u_short		no_of_marks;	/* This many marked texts */
	u_short		no_of_confs;	/* Member in this many confs */
#else	/* SERVER */
	Text_list	created_texts;
	Mark_list	marks;		/* List of marked texts */
	Membership_list conferences;	/* List of conferences the person is
					 * a member in. */
	Password	pwd;		/* Encrypted password */
#endif
} Person;

#ifdef SERVER
#	define EMPTY_PERSON_i	((Person)			\
	{	0, DEFAULT_PRIV_BITS_i,				\
		DEFAULT_PERSONAL_FLAGS_i, NULL_TIME_i, 0, 0,	\
		0, 0, 0, 0, 0, 0, EMPTY_STRING_i,		\
		EMPTY_TEXT_LIST_i, EMPTY_MARK_LIST_i,		\
		EMPTY_MEMBERSHIP_LIST_i, "\001password"		\
	})
#else
#	define EMPTY_PERSON_i	((Person)			\
	{	0, DEFAULT_PRIV_BITS_i,				\
		DEFAULT_PERSONAL_FLAGS_i, NULL_TIME_i, 0, 0,	\
		0, 0, 0, 0, 0, 0, EMPTY_STRING_i, 0, 0, 0, 0	\
	})
#endif
extern  const Person	EMPTY_PERSON;



/*
 * This struct is returned from the 'who_is_on' call.
 */
typedef struct {
	Pers_no		person;
	String		what_am_i_doing;
	Conf_no		working_conference;
} Who_info_old;

typedef struct {
	Pers_no		person;
	String		what_am_i_doing;
	String		username;           /* Userid and hostname. */
	Conf_no		working_conference;
	Session_no	session_no;         /* Serial number of connection. */
} Who_info;

typedef struct {
	Pers_no		person;
	String		what_am_i_doing;
	String		username;            /* Userid and hostname. */
	Conf_no		working_conference;
	Session_no	session;             /* Serial number of connection. */
	Time		connection_time;     /* Not logintime. */
	u_long		idle_time; 	     /* Seconds. */
} Session_info;

#define EMPTY_SESSION_INFO_i	((Session_info) { 0, EMPTY_STRING_i,    \
						  EMPTY_STRING_i, 0, 0, \
						  NULL_TIME_i, 0 })

extern const Session_info	EMPTY_SESSION_INFO;

#define	EMPTY_WHO_INFO_OLD_i	((Who_info_old) {  0, EMPTY_STRING_i, 0 })

#define	EMPTY_WHO_INFO_i	((Who_info) {  0, EMPTY_STRING_i, \
						   EMPTY_STRING_i, 0, 0 })

extern const Who_info_old	EMPTY_WHO_INFO_OLD;

extern const Who_info		EMPTY_WHO_INFO;



typedef struct {
	int		  no_of_persons;
    	Who_info_old	* info;
} Who_info_list_old;	

#define EMPTY_WHO_INFO_LIST_OLD_i ((Who_info_list_old) {  0, NULL  })
extern const Who_info_list_old	EMPTY_WHO_INFO_LIST_OLD;


typedef struct {
	int		  no_of_persons;
    	Who_info	* info;
} Who_info_list;	

#define EMPTY_WHO_INFO_LIST_i	((Who_info_list) {  0, NULL  })
extern const Who_info_list	EMPTY_WHO_INFO_LIST;

#define EMPTY_tm_i		0, 0, 0, 0, 0, 0, 0, 0, 0, NULL, 0

#endif	/*  ifndef  KOM_TYPES_H_ALREADY_INCLUDED__  */
